<?php

namespace Botble\Documents\Http\Controllers;

use Botble\ACL\Models\User;
use Botble\Base\Events\BeforeEditContentEvent;
use Botble\Base\Events\CreatedContentEvent;
use Botble\Base\Events\DeletedContentEvent;
use Botble\Base\Events\UpdatedContentEvent;
use Botble\Base\Forms\FormBuilder;
use Botble\Base\Http\Controllers\BaseController;
use Botble\Base\Http\Responses\BaseHttpResponse;
use Botble\Base\Traits\HasDeleteManyItemsTrait;
//use Botble\Documents\Forms\PostForm;
use Botble\Documents\Forms\DocumentForm;
//use Botble\Documents\Http\Requests\PostRequest;
use Botble\Documents\Http\Requests\DocumentRequest;
//use Botble\Documents\Models\Post;
use Botble\Documents\Models\Document;
use Botble\Documents\Repositories\Interfaces\CategoryInterface;
//use Botble\Documents\Repositories\Interfaces\PostInterface;
use Botble\Documents\Repositories\Interfaces\DocumentInterface;
use Botble\Documents\Repositories\Interfaces\TagInterface;
use Botble\Documents\Services\StoreCategoryService;
use Botble\Documents\Services\StoreTagService;
//use Botble\Documents\Tables\PostTable;
use Botble\Documents\Tables\DocumentTable;
use Exception;
use Illuminate\Contracts\View\Factory;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\View\View;
use Throwable;

class DocumentController extends BaseController
{

    use HasDeleteManyItemsTrait;

    /**
     * @var PostInterface
     */
    protected $documentRepository;

    /**
     * @var TagInterface
     */
    protected $tagRepository;

    /**
     * @var CategoryInterface
     */
    protected $categoryRepository;

    /**
     * @param PostInterface $postRepository
     * @param TagInterface $tagRepository
     * @param CategoryInterface $categoryRepository
     */
    public function __construct(
        DocumentInterface $documentRepository,
        TagInterface $tagRepository,
        CategoryInterface $categoryRepository
    ) {
        $this->documentRepository = $documentRepository;
        $this->tagRepository = $tagRepository;
        $this->categoryRepository = $categoryRepository;
    }

    /**
     * @param PostTable $dataTable
     * @return Factory|View
     * @throws Throwable
     */
    public function index(DocumentTable $dataTable)
    {
        page_title()->setTitle(trans('plugins/documents::documentsposts.menu_name'));

        return $dataTable->renderTable();
    }

    /**
     * @param FormBuilder $formBuilder
     * @return string
     */
    public function create(FormBuilder $formBuilder)
    {
        page_title()->setTitle(trans('plugins/documents::documentsposts.create'));

        return $formBuilder->create(DocumentForm::class)->renderForm();
    }

    /**
     * @param PostRequest $request
     * @param StoreTagService $tagService
     * @param StoreCategoryService $categoryService
     * @param BaseHttpResponse $response
     * @return BaseHttpResponse
     */
    public function store(
        DocumentRequest $request,
        StoreTagService $tagService,
        StoreCategoryService $categoryService,
        BaseHttpResponse $response
    ) {
        /**
         * @var Post $post
         */

      //  dd($request);
        $document = $this->documentRepository->createOrUpdate(array_merge($request->input(), [
            'author_id'   => Auth::id(),
            'author_type' => User::class,
        ]));
        

   /*    $file = $request->file('uploaddocument');
     $path = 'public/images/'.$file;//$file->store('images', ['disk' =>   'local']);
       $file_name = $request->file('uploaddocument')->move('public/images', $path);*/
      
      /* $file[] = $request->file('uploaddocument');
       $fileName = $file->getClientOriginalName(); // Get the original file name
       $path = $file->move(public_path('uploaddocument'), $fileName); // Move the file to the desired location
        */
        
         $files = $request->file('uploaddocument');
        foreach ($files as $file) {
            $fileName = $file->getClientOriginalName(); // Get the original file name
            $file->move(public_path('uploaddocument'), $fileName); // Move each file to the desired location
        }

      print_r( $files );
     exit;

        event(new CreatedContentEvent(DOCUMENT_MODULE_SCREEN_NAME, $request, $document));

        $tagService->execute($request, $document);

        $categoryService->execute($request, $document);

        return $response
            ->setPreviousUrl(route('documentsposts.index'))
            ->setNextUrl(route('documentsposts.edit', $document->id))
            ->setMessage(trans('core/base::notices.create_success_message'));

    }

    /**
     * @param int $id
     * @param FormBuilder $formBuilder
     * @param Request $request
     * @return string
     */
    public function edit($id, FormBuilder $formBuilder, Request $request)
    {
        $document = $this->documentRepository->findOrFail($id);

        event(new BeforeEditContentEvent($request, $document));

        page_title()->setTitle(trans('plugins/documents::documentsposts.edit') . ' "' . $document->name . '"');

        return $formBuilder->create(DocumentForm::class, ['model' => $document])->renderForm();
    }

    /**
     * @param int $id
     * @param PostRequest $request
     * @param StoreTagService $tagService
     * @param StoreCategoryService $categoryService
     * @param BaseHttpResponse $response
     * @return BaseHttpResponse
     */
    public function update(
        $id,
        DocumentRequest $request,
        StoreTagService $tagService,
        StoreCategoryService $categoryService,
        BaseHttpResponse $response
    ) {

       // dd($request);
        $document = $this->documentRepository->findOrFail($id);

        $document->fill($request->input());

        $this->documentRepository->createOrUpdate($document);

        event(new UpdatedContentEvent(DOCUMENT_MODULE_SCREEN_NAME, $request, $document));

        $tagService->execute($request, $document);

        $categoryService->execute($request, $document);

        return $response
            ->setPreviousUrl(route('documentsposts.index'))
            ->setMessage(trans('core/base::notices.update_success_message'));
    }

    /**
     * @param int $id
     * @param Request $request
     * @return BaseHttpResponse
     */
    public function destroy($id, Request $request, BaseHttpResponse $response)
    {
        try {
            $document = $this->documentRepository->findOrFail($id);
            $this->documentRepository->delete($document);

            event(new DeletedContentEvent(DOCUMENT_MODULE_SCREEN_NAME, $request, $document));

            return $response
                ->setMessage(trans('core/base::notices.delete_success_message'));
        } catch (Exception $exception) {
            return $response
                ->setError()
                ->setMessage($exception->getMessage());
        }
    }

    /**
     * @param Request $request
     * @param BaseHttpResponse $response
     * @return BaseHttpResponse
     * @throws Exception
     */
    public function deletes(Request $request, BaseHttpResponse $response)
    {
        return $this->executeDeleteItems($request, $response, $this->documentRepository, DOCUMENT_MODULE_SCREEN_NAME);
    }

    /**
     * @param Request $request
     * @param BaseHttpResponse $response
     * @return BaseHttpResponse
     * @throws Throwable
     */
    public function getWidgetRecentDocuments(Request $request, BaseHttpResponse $response)
    {
        $limit = (int)$request->input('paginate', 10);
        $limit = $limit > 0 ? $limit : 10;

        $documents = $this->documentsRepository->advancedGet([
            'with'     => ['slugable'],
            'order_by' => ['created_at' => 'desc'],
            'paginate' => [
                'per_page'      => $limit,
                'current_paged' => (int)$request->input('page', 1),
            ],
        ]);

        return $response
            ->setData(view('plugins/documents::documentsposts.widgets.documentsposts', compact('documentsposts', 'limit'))->render());
    }
}
