<?php

namespace Botble\Careers\Listeners;

use Botble\Careers\Repositories\Interfaces\CategoryInterface;
use Botble\Careers\Repositories\Interfaces\CareerInterface;
use Botble\Careers\Repositories\Interfaces\TagInterface;
use SiteMapManager;

class RenderingSiteMapListener
{
    /**
     * @var PostInterface
     */
    protected $careerRepository;

    /**
     * @var CategoryInterface
     */
    protected $categoryRepository;

    /**
     * @var TagInterface
     */
    protected $tagRepository;

    /**
     * RenderingSiteMapListener constructor.
     * @param PostInterface $postRepository
     * @param CategoryInterface $categoryRepository
     * @param TagInterface $tagRepository
     */
    public function __construct(
        CareerInterface $careerRepository,
        CategoryInterface $categoryRepository,
        TagInterface $tagRepository
    ) {
        $this->careerRepository = $careerRepository;
        $this->categoryRepository = $categoryRepository;
        $this->tagRepository = $tagRepository;
    }

    /**
     * Handle the event.
     *
     * @return void
     */
    public function handle()
    {
        $careers = $this->careerRepository->getDataSiteMap();

        foreach ($careers as $career) {
            SiteMapManager::add($career->url, $career->updated_at, '0.8');
        }

        $categories = $this->categoryRepository->getDataSiteMap();

        foreach ($categories as $category) {
            SiteMapManager::add($category->url, $category->updated_at, '0.8');
        }

        $tags = $this->tagRepository->getDataSiteMap();

        foreach ($tags as $tag) {
            SiteMapManager::add($tag->url, $tag->updated_at, '0.3', 'weekly');
        }
    }
}
