<?php

namespace Botble\Careers\Providers;

use Botble\LanguageAdvanced\Supports\LanguageAdvancedManager;
use Botble\Shortcode\View\View;
use Illuminate\Routing\Events\RouteMatched;
use Botble\Base\Traits\LoadAndPublishDataTrait;
use Botble\Careers\Models\Career;
use Botble\Careers\Repositories\Caches\CareerCacheDecorator;
use Botble\Careers\Repositories\Eloquent\CareerRepository;
use Botble\Careers\Repositories\Interfaces\CareerInterface;
use Illuminate\Support\Facades\Event;
use Illuminate\Support\ServiceProvider;
use Botble\Careers\Models\Category;
use Botble\Careers\Repositories\Caches\CategoryCacheDecorator;
use Botble\Careers\Repositories\Eloquent\CategoryRepository;
use Botble\Careers\Repositories\Interfaces\CategoryInterface;
use Botble\Careers\Models\Tag;
use Botble\Careers\Repositories\Caches\TagCacheDecorator;
use Botble\Careers\Repositories\Eloquent\TagRepository;
use Botble\Careers\Repositories\Interfaces\TagInterface;
use Language;
use Note;
use SeoHelper;
use SlugHelper;

/**
 * @since 02/07/2016 09:50 AM
 */
class CareersServiceProvider extends ServiceProvider
{
    use LoadAndPublishDataTrait;

    public function register()
    {
        $this->app->bind(CareerInterface::class, function () {
            return new CareerCacheDecorator(new CareerRepository(new Career));
        });
         

        $this->app->bind(CategoryInterface::class, function () {
            return new CategoryCacheDecorator(new CategoryRepository(new Category));
        });

        $this->app->bind(TagInterface::class, function () {
            return new TagCacheDecorator(new TagRepository(new Tag));
        });
    }

    public function boot()
    {
        SlugHelper::registerModule(Career::class, 'Careers Posts');
        SlugHelper::registerModule(Category::class, 'Careers Categories');
        SlugHelper::registerModule(Tag::class, 'Careers Tags');

        SlugHelper::setPrefix(Tag::class, 'tag', true);
        SlugHelper::setPrefix(Career::class, null, true);
        SlugHelper::setPrefix(Category::class, null, true);

        $this->setNamespace('plugins/careers')
            ->loadHelpers()
            ->loadAndPublishConfigurations(['permissions', 'general'])
            ->loadAndPublishViews()
            ->loadAndPublishTranslations()
            ->loadRoutes(['web', 'api'])
            ->loadMigrations()
            ->publishAssets();

         $this->app->register(EventServiceProvider::class);

        Event::listen(RouteMatched::class, function () {
            dashboard_menu()
                ->registerItem([
                    'id'          => 'cms-plugins-careers',
                    'priority'    => 3,
                    'parent_id'   => null,
                    'name'        => 'Careers',
                    'icon'        => 'fa fa-graduation-cap',
                    //'url'       => route('posts.index'),
                    'url'         => url("admin/careers/careersposts"),
                    'permissions' => ['careersposts.index'],
                ])
                ->registerItem([
                    'id'          => 'cms-plugins-careers-careersposts',
                    'priority'    => 1,
                    'parent_id'   => 'cms-plugins-careers',
                    'name'        => 'Careers Management',
                    'icon'        => null,
                    //'url'         => route('posts.index'),
                    'url'         => url("admin/careers/careersposts"),
                    'permissions' => ['careersposts.index'],
                ])
                ->registerItem([
                    'id'          => 'cms-plugins-careers-categories',
                    'priority'    => 2,
                    'parent_id'   => 'cms-plugins-careers',
                    'name'        => 'Manage Categories',
                    'icon'        => null,
                    //'url'         => route('categories.index'),
                    'url'         => url("admin/careers/careerscategories"),
                    'permissions' => ['careerscategories.index'],
                ])
                // ->registerItem([
                //     'id'          => 'cms-plugins-careers-tags',
                //     'priority'    => 3,
                //     'parent_id'   => 'cms-plugins-careers',
                //     'name'        => 'plugins/careers::tags.menu_name',
                //     'icon'        => null,
                //     'url'         => url("admin/careers/careerstags"),
                //     //'url'         => route('tags.index'),
                //     'permissions' => ['careerstags.index'],
                // ])
                ;
        });

        $useLanguageV2 = $this->app['config']->get('plugins.careers.general.use_language_v2', false) &&
            defined('LANGUAGE_ADVANCED_MODULE_SCREEN_NAME');

        if (defined('LANGUAGE_MODULE_SCREEN_NAME') && $useLanguageV2) {
            LanguageAdvancedManager::registerModule(Career::class, [
                'name',
                'description',
                'content',
            ]);

            LanguageAdvancedManager::registerModule(Category::class, [
                'name',
                'description',
            ]);

            LanguageAdvancedManager::registerModule(Tag::class, [
                'name',
                'description',
            ]);
        }

        $this->app->booted(function () use ($useLanguageV2) {
            $models = [Career::class, Category::class, Tag::class];

            if (defined('LANGUAGE_MODULE_SCREEN_NAME') && !$useLanguageV2) {
                Language::registerModule($models);
            }

            SeoHelper::registerModule($models);

            $configKey = 'packages.revision.general.supported';
            config()->set($configKey, array_merge(config($configKey, []), [Career::class]));

            if (defined('NOTE_FILTER_MODEL_USING_NOTE')) {
                Note::registerModule(Career::class);
            }

            $this->app->register(HookServiceProvider::class);
        });

        if (function_exists('shortcode')) {
            view()->composer([
                'plugins/careers::themes.careers',
                'plugins/careers::themes.careers',
                'plugins/careers::themes.tag',
            ], function (View $view) {
                $view->withShortcodes();
            });
        }
    }
}
