<?php

namespace Botble\Careers\Repositories\Eloquent;

use Botble\Base\Enums\BaseStatusEnum;
use Botble\Careers\Repositories\Interfaces\CareerInterface;
use Botble\Support\Repositories\Eloquent\RepositoriesAbstract;
use Eloquent;
use Exception;
use Illuminate\Database\Query\Builder;
use Illuminate\Support\Arr;
use Illuminate\Support\Facades\DB;
use Illuminate\Pagination\LengthAwarePaginator;
use Illuminate\Pagination\Paginator;

class CareerRepository extends RepositoriesAbstract implements CareerInterface
{
    /**
     * {@inheritDoc}
     */
    public function getFeatured(int $limit = 5, array $with = [])
    {
        $data = $this->model
            ->where([
                'status'      => BaseStatusEnum::PUBLISHED,
                'is_featured' => 1,
            ])
            ->limit($limit)
            ->with(array_merge(['slugable'], $with))
            ->orderBy('created_at', 'desc');

        return $this->applyBeforeExecuteQuery($data)->get();
    }

    /**
     * {@inheritDoc}
     */
    public function getListCareerNonInList(array $selected = [], $limit = 7, array $with = [])
    {
        $data = $this->model
            ->where('status', BaseStatusEnum::PUBLISHED)
            ->whereNotIn('id', $selected)
            ->limit($limit)
            ->with($with)
            ->orderBy('created_at', 'desc');

        return $this->applyBeforeExecuteQuery($data)->get();
    }

    /**
     * {@inheritDoc}
     */
    public function getRelated($id, $limit = 3)
    {
       $catarrays = array();
       $cat = 0;
       $catrry = $this->getRelatedCategoryIds($id);
       foreach($catrry as $cats)
       { 
            $cat = $cats;
       }
       $catarrays[] = $cat;
      //print_r($catarrays);
       $data = $this->model
            ->where('status', BaseStatusEnum::PUBLISHED)
            ->where('id', '!=', $id)
            ->limit($limit)
            ->with('slugable')
            ->orderBy('created_at', 'desc')
            ->whereHas('newcategories', function ($query) use ($catarrays) {
                //$catarray[] =  $this->getRelatedCategoryIds($id)[1];
                $query->whereIn('newcategories.id', $catarrays);
            });
     
        return $this->applyBeforeExecuteQuery($data)->get();
    }

    /**
     * {@inheritDoc}
     */
    public function getRelatedCategoryIds($model)
    {
        $model = $model instanceof Eloquent ? $model : $this->findById($model);

        if (!$model) {
            return [];
        }

        try {
            return $model->categories()->allRelatedIds()->toArray();
        } catch (Exception $exception) {
            return [];
        }
    }

    /**
     * {@inheritDoc}
     */
    public function getByCategory($categoryId, $paginate = 12, $limit = 0)
    {
        if (!is_array($categoryId)) {
            $categoryId = [$categoryId];
        }

        $data = $this->model
            ->where('careers.status', BaseStatusEnum::PUBLISHED)
            ->join('careers_categories', 'careers_categories.career_id', '=', 'careers.id')
            ->join('newcategories', 'careers_categories.category_id', '=', 'newcategories.id')
            ->whereIn('careers_categories.category_id', $categoryId)
            ->select('careers.*')
            ->distinct()
            ->with('slugable')
            ->orderBy('careers.created_at', 'desc');

        if ($paginate != 0) {
            return $this->applyBeforeExecuteQuery($data)->paginate($paginate);
        }

        return $this->applyBeforeExecuteQuery($data)->limit($limit)->get();
    }

    /**
     * {@inheritDoc}
     */
    public function getByUserId($authorId, $paginate = 6)
    {
        $data = $this->model
            ->where([
                'status'    => BaseStatusEnum::PUBLISHED,
                'author_id' => $authorId,
            ])
            ->with('slugable')
            ->orderBy('created_at', 'desc');

        return $this->applyBeforeExecuteQuery($data)->paginate($paginate);
    }

    /**
     * {@inheritDoc}
     */
    public function getDataSiteMap()
    {
        $data = $this->model
            ->with('slugable')
            ->where('status', BaseStatusEnum::PUBLISHED)
            ->orderBy('created_at', 'desc');

        return $this->applyBeforeExecuteQuery($data)->get();
    }

    /**
     * {@inheritDoc}
     */
    public function getByTag($tag, $paginate = 12)
    {
        $data = $this->model
            ->with('slugable', 'newcategories', 'newcategories.slugable', 'author')
            ->where('status', BaseStatusEnum::PUBLISHED)
            ->whereHas('tags', function ($query) use ($tag) {
                /**
                 * @var Builder $query
                 */
                $query->where('tags.id', $tag);
            })
            ->orderBy('created_at', 'desc');

        return $this->applyBeforeExecuteQuery($data)->paginate($paginate);
    }

    /**
     * {@inheritDoc}
     */
    public function getRecentCareer($limit = 5, $categoryId = 0)
    {
        $data = $this->model->where(['status' => BaseStatusEnum::PUBLISHED]);

        // $categories = DB::select('select * from post_categories');

        // echo '<pre>';
        // print_r($categoryId);
        // die;

        if ($categoryId != 0) {
            $data = $data
                ->join('careers_categories', 'careers_categories.career_id', '=', 'careers.id')
                ->where('careers_categories.category_id', $categoryId);
        }

        $data = $data->limit($limit)
            ->with('slugable')
            ->select('careers.*')
            ->orderBy('created_at', 'desc');

        return $this->applyBeforeExecuteQuery($data)->get();
    }

    /**
     * {@inheritDoc}
     */
    public function getSearch($keyword, $limit = 10, $paginate = 10)
    {
        $data = $this->model
            ->with('slugable')
            ->where('status', BaseStatusEnum::PUBLISHED)
            ->where(function ($query) use ($keyword) {
                $query->addSearch('name', $keyword);
            })
            ->orderBy('created_at', 'desc');

        if ($limit) {
            $data = $data->limit($limit);
        }

        if ($paginate) {
            return $this->applyBeforeExecuteQuery($data)->paginate($paginate);
        }

        return $this->applyBeforeExecuteQuery($data)->get();
    }

    /**
     * {@inheritDoc}
     */
    public function getAllCareer($perPage = 12, $active = true, array $with = ['slugable'])
    {
        $data = $this->model
            ->with($with)
            ->orderBy('created_at', 'desc');

        if ($active) {
            $data = $data->where('status', BaseStatusEnum::PUBLISHED);
        }

        return $this->applyBeforeExecuteQuery($data)->paginate($perPage);
    }

    /**
     * {@inheritDoc}
     */
    public function getPopularCareer($limit, array $args = [])
    {
        $data = $this->model
            ->with('slugable')
            ->orderBy('views', 'desc')
            ->where('status', BaseStatusEnum::PUBLISHED)
            ->limit($limit);

        if (!empty(Arr::get($args, 'where'))) {
            $data = $data->where($args['where']);
        }

        return $this->applyBeforeExecuteQuery($data)->get();
    }

    /**
     * {@inheritDoc}
     */
    public function getFilters(array $filters)
    {
        $data = $this->originalModel;

        if ($filters['categories'] !== null) {
            $categories = array_filter((array)$filters['categories']);

            $data = $data->whereHas('categories', function ($query) use ($categories) {
                $query->whereIn('id', $categories);
            });
        }

        if ($filters['categories_exclude'] !== null) {
            $data = $data
                ->whereHas('categories', function ($query) use ($filters) {
                    $query->whereNotIn('id', array_filter((array)$filters['categories_exclude']));
                });
        }

        if ($filters['exclude'] !== null) {
            $data = $data->whereNotIn('id', array_filter((array)$filters['exclude']));
        }

        if ($filters['include'] !== null) {
            $data = $data->whereNotIn('id', array_filter((array)$filters['include']));
        }

        if ($filters['author'] !== null) {
            $data = $data->whereIn('author_id', array_filter((array)$filters['author']));
        }

        if ($filters['author_exclude'] !== null) {
            $data = $data->whereNotIn('author_id', array_filter((array)$filters['author_exclude']));
        }

        if ($filters['featured'] !== null) {
            $data = $data->where('is_featured', $filters['featured']);
        }

        if ($filters['search'] !== null) {
            $data = $data
                ->where(function ($query) use ($filters) {
                    $query
                        ->addSearch('name', $filters['search'])
                        ->addSearch('description', $filters['search']);
                });
        }

        $orderBy = Arr::get($filters, 'order_by', 'updated_at');
        $order = Arr::get($filters, 'order', 'desc');

        $data = $data
            ->where('status', BaseStatusEnum::PUBLISHED)
            ->orderBy($orderBy, $order);

        return $this->applyBeforeExecuteQuery($data)->paginate((int)$filters['per_page']);
    }

    public function getRecentByCategory($categoryId,$perPage=0,$limit = 0)
    {         
      // dd($categoryId);
        $search = '';
        if(isset($_GET['qsearch']) && $_GET['qsearch']!=''){
           $search = $_GET['qsearch'];    
        }
        
        $tag_id = '';
        if(isset($_GET['tag']) && $_GET['tag']!=''){
           $tag_id = $_GET['tag'];    
        }
        
        $archiveY = '';
        if(isset($_GET['archiveY']) && $_GET['archiveY']!=''){
           $archiveY = $_GET['archiveY'];    
        }
        
        $archiveM = '';
        if(isset($_GET['archiveM']) && $_GET['archiveM']!=''){
           $archiveM = $_GET['archiveM'];    
        }
        
     
       // exit();
        if($limit==0) {
            $limit=-1;
        }
        
          if($perPage==0) {
             if($categoryId['0'] == '129' || $categoryId['0'] == '127' || $categoryId['0'] == '125' || $categoryId['0'] == '126' || $categoryId['0'] == '116' || $categoryId['0'] == '114') { 
              $perPage = 12;
             } else {
                 $perPage = 6;
             }
          } 
        

         $tag_pag = '';
      
       \DB::enableQueryLog(); // Enable query log
      // if($tag_id == ''){
        $data = $this->model
            ->where('careers.status', BaseStatusEnum::PUBLISHED)
            ->join('careers_categories', 'careers_categories.career_id', '=', 'careers.id')
            ->join('newcategories', 'newcategories.id', '=', 'careers_categories.category_id')
            //->join('post_tags', 'post_tags.post_id', '=', 'posts.id')
            ->when(isset($_GET['tag']) , function ($data) {
                  $data->join('careers_tags', 'careers_tags.career_id', '=', 'careers.id');
                    $data->Where('careers_tags.tag_id', '=',  $_GET['tag'] );
                })
             ->when(isset($_GET['archiveM']) , function ($data) {
                    $data->whereMonth('careers.created_at', '=',$_GET['archiveM']);
                    $data->whereYear('careers.created_at', '=', $_GET['archiveY']);
                })  
            ->Where('careers.name', 'like', '%' . $search . '%')
           // ->Where('post_tags.tag_id', '=',  $tag_id )
            //->whereIn('categories.parent_id', array_filter((array)$categoryId))
            ->WhereIn('categories.id', array_filter((array)$categoryId))
            ->select('careers.*','categories.name as cat_name')            
            ->distinct()
            ->limit($limit)   
            ->with('slugable')
            ->groupBy('careers.id')
            ->orderBy('careers.custom_ipdate', 'desc')
            ->orderBy('careers.custom_post_inslider', 'desc')
            ->orderBy('careers.testimonial_video_url', 'desc')
            ->orderBy('careers.uploaddocument', 'desc')            
            ->orderBy('careers.custom_iptime', 'desc')->get();
       dd(\DB::getQueryLog()); // Show results of log
            $totalGroup = count($data);            
            $page = Paginator::resolveCurrentPage('page');
         //   $page2 = Paginator::resolveCurrentPage('qsearch');
       
         if(isset($_GET['tag'])) {
                  $tag_pag = $_GET['tag'];
          $data = new LengthAwarePaginator($data->forPage($page, $perPage), $totalGroup, $perPage, $page, [
             'path' => Paginator::resolveCurrentPath().'?tag='.$tag_pag,
             'pageName' => 'page',
            ]);       
        } else {
             $data = new LengthAwarePaginator($data->forPage($page, $perPage), $totalGroup, $perPage, $page, [
             'path' => Paginator::resolveCurrentPath(),
             'pageName' => 'page',
            ]);
            
        }
            //dd($data); 
             
            return $data;        
          //  return $this->applyBeforeExecuteQuery($data)->limit($limit)->get();
    }

    public function getNextCareer($categoryId,$careersId)
    {                
        //echo $categoryId.'------'.$postId;
        $data = $this->model
            ->where('careers.status', BaseStatusEnum::PUBLISHED)
            ->where('careers_categories.category_id','=', $categoryId)
            ->where('careers.id','>', $careersId)
            ->join('careers_categories', 'careers_categories.career_id', '=', 'careers.id')                                    
            ->select('careers.*')   
            ->limit(1)                     
            ->with('slugable')
            ->orderBy('careers.created_at', 'desc')->get();
            //dd($data); 
        
        //return $this->applyBeforeExecuteQuery($data)->paginate(3);    
        return $data;//$this->applyBeforeExecuteQuery($data)->limit($limit)->get();
    }
    
    public function getPrevCareer($categoryId,$careersId)
    {                
        //echo $categoryId.'------'.$postId;
        $data = $this->model
            ->where('careers.status', BaseStatusEnum::PUBLISHED)
            ->where('careers_categories.category_id','=', $categoryId)
            ->where('careers.id','<', $careersId)
            ->join('careers_categories', 'careers_categories.career_id', '=', 'careers.id')                                    
            ->select('careers.*')   
            ->limit(1)                     
            ->with('slugable')
            ->orderBy('careers.created_at', 'asc')->get();
            //dd($data); 
        
        //return $this->applyBeforeExecuteQuery($data)->paginate(3);    
        return $data;//$this->applyBeforeExecuteQuery($data)->limit($limit)->get();
    }
    public function getcareer($careersId)
    {                
        $data = $this->model
            ->where('careers.id','=', $careersId)                                
            ->select('careers.*')   
            ->limit(1)                     
            ->get();
           
        return $data;
    }

}
