<?php

namespace Botble\Careers\Forms;

use Botble\Base\Enums\BaseStatusEnum;
use Botble\Base\Enums\EmployeeTypeEnum;
use Botble\Base\Forms\Fields\TagField;
use Botble\Base\Forms\FormAbstract;
use Botble\Careers\Forms\Fields\CategoryMultiField;
// use Botble\Careers\Forms\Fields\NewCategoryMultiField;
use Botble\Careers\Http\Requests\CareerRequest;
use Botble\Careers\Models\Career;
use Botble\Careers\Repositories\Interfaces\CategoryInterface;

class CareerForm extends FormAbstract
{

    /**
     * @var string
     */
    protected $template = 'core/base::forms.form-tabs';

    /**
     * {@inheritDoc}
     * @throws \Exception
     */
    public function buildForm()
    {
        $selectedCategories = [];

        if ($this->getModel()) {
            $selectedCategories = $this->getModel()->categories()->pluck('category_id')->all();
        }

        if (empty($selectedCategories)) {
            $selectedCategories = app(CategoryInterface::class)
                ->getModel()
                ->where('is_default', 1)
                ->pluck('id')
                ->all();
        }

        $tags = null;

        if ($this->getModel()) {
            $tags = $this->getModel()->tags()->pluck('name')->all();
            $tags = implode(',', $tags);
        }

        if (!$this->formHelper->hasCustomField('categoryMulti')) {
            $this->formHelper->addCustomField('categoryMulti', CategoryMultiField::class);
        }

        // if (!$this->formHelper->hasCustomField('newcategoryMulti')) {
        //     $this->formHelper->addCustomField('newcategoryMulti', NewCategoryMultiField::class);
        // }

        $this
            ->setupModel(new Career)
            ->setValidatorClass(CareerRequest::class)
            ->withCustomFields()
            ->addCustomField('
                tags', TagField::class)
            ->add('name', 'text', [
                'label'      => trans('Job Title'),
                'label_attr' => ['class' => 'control-label required'],
                'attr'       => [
                    'placeholder'  => trans('Job Title'),
                ],
            ])
            ->add('department', 'text', [
                'label'      => trans('Department'),
                'label_attr' => ['class' => 'control-label '],
                'attr'       => [
                    'placeholder'  => trans('Department'),
                ],
            ])
            ->add('education', 'text', [
                'label'      => trans('Education'),
                'label_attr' => ['class' => 'control-label '],
                'attr'       => [
                    'placeholder'  => trans('Education'),
                ],
            ])
            ->add('role', 'text', [
                'label'      => trans('Role'),
                'label_attr' => ['class' => 'control-label '],
                'attr'       => [
                    'placeholder'  => trans('Role'),
                ],
            ])
            ->add('experience', 'text', [
                'label'      => trans('Experience'),
                'label_attr' => ['class' => 'control-label '],
                'attr'       => [
                    'placeholder'  => trans('Experience'),
                ],
            ])

            ->add('employmenttype', 'customSelect', [
                'label'      => trans('Employment Type'),
                'label_attr' => ['class' => 'control-label '],
               // 'choices'    => EmployeeTypeEnum::labels(),
            ])

            ->add('location', 'text', [
                'label'      => trans('Location'),
                'label_attr' => ['class' => 'control-label '],
                'attr'       => [
                    'placeholder'  => trans('Location'),
                ],
            ])

            ->add('knowledge', 'editor', [
                'label'      => trans('Add Knowledge'),
                'label_attr' => ['class' => 'control-label '],
                'attr'       => [
                    'rows'         => 4,
                    'placeholder'  => trans('Add Knowledge'),
                    'with-short-code' => true,
                ],
            ])

            ->add('skills', 'editor', [
                'label'      => trans('Key Skills'),
                'label_attr' => ['class' => 'control-label '],
                'attr'       => [
                    'rows'         => 4,
                    'placeholder'  => trans('Enter key skills separated by commas(,)'),
                    'with-short-code' => true,
                ],
            ])

            ->add('description', 'editor', [
                'label'      => trans('core/base::forms.description'),
                'label_attr' => ['class' => 'control-label '],
                'attr'       => [
                    'rows'         => 4,
                    'placeholder'  => trans('core/base::forms.description_placeholder'),
                    'with-short-code' => true,
                ],
            ])

            ->add('responsibilities', 'editor', [
                'label'      => trans('Responsibilities'),
                'label_attr' => ['class' => 'control-label '],
                'attr'       => [
                    'rows'         => 4,
                    'placeholder'  => trans('Responsibilities'),
                    'with-short-code' => true,
                ],
            ])

            ->add('requirements', 'editor', [
                'label'      => trans('Requirements'),
                'label_attr' => ['class' => 'control-label '],
                'attr'       => [
                    'rows'         => 4,
                    'placeholder'  => trans('Requirements'),
                    'with-short-code' => true,
                ],
            ])
            // ->add('is_featured', 'onOff', [
            //     'label'         => trans('core/base::forms.is_featured'),
            //     'label_attr'    => ['class' => 'control-label'],
            //     'default_value' => false,
            // ])
            // ->add('content', 'editor', [
            //     'label'      => trans('core/base::forms.content'),
            //     'label_attr' => ['class' => 'control-label'],
            //     'attr'       => [
            //         'rows'            => 4,
            //         'placeholder'     => trans('core/base::forms.description_placeholder'),
            //         'with-short-code' => true,
            //     ],
            // ])
            
            ->add('status', 'customSelect', [
                'label'      => trans('core/base::tables.status'),
                'label_attr' => ['class' => 'control-label '],
                'choices'    => BaseStatusEnum::labels(),
            ])   

            // ->add('categories[]', 'categoryMulti', [
            //     'label'      => trans('categories'),
            //     'label_attr' => ['class' => 'control-label '],
            //     'choices'    => get_categories_with_children(),
            //     'value'      => old('categories', $selectedCategories),
            // ]) 

            ->add('categories[]', 'categoryMulti', [
                'label'      => trans('Categories'),
                'label_attr' => ['class' => 'control-label '],
                'choices'    => get_newcategories_with_children(),
                'value'      => old('categories', $selectedCategories),
            ]) 

            // ->add('uploaddocument[]', 'file', [
            //     'label'      => 'upload File',//trans('core/base::forms.uploaddocument'),
            //     'label_attr' => ['class' => 'control-label'],
            //     'attr' => ['multiple'],
                
            //    // 'attributes' => ['multiple' => true], // Add the 'multiple' attribute to enable multiple file section

            // ])
            // ->add('image', 'mediaImage', [
            //     'label'      => trans('core/base::forms.image'),
            //     'label_attr' => ['class' => 'control-label'],
            // ])
            // ->add('tag', 'tags', [
            //     'label'      => trans('plugins/careers::careersposts.form.tags'),
            //     'label_attr' => ['class' => 'control-label'],
            //     'value'      => $tags,
            //     'attr'       => [
            //         'placeholder' => trans('plugins/careers::base.write_some_tags'),
            //         'data-url'    => route('tags.all'),
            //     ],
            // ])
            
            /*   ->add('publisher', 'publishers', [
                'label'      => trans('plugins/blog::posts.form.publishers'),
                'label_attr' => ['class' => 'control-label'],
                'value'      => $tags,
                'attr'       => [
                    'placeholder' => trans('plugins/blog::base.write_some_tags'),
                    'data-url'    => route('tags.all'),
                ],
            ])*/
            
            ->setBreakFieldPoint('status');

        $careersFormats = get_careersposts_formats(true);

        if (count($careersFormats) > 1) {
            $this->addAfter('status', 'format_type', 'customRadio', [
                'label'      => trans('plugins/careers::careers.form.format_type'),
                'label_attr' => ['class' => 'control-label'],
                'choices'    => get_careersposts_formats(true),
            ]);
        /*   $this->addAfter('image', 'uploaddocument', 'file', [
                'label'      => 'upload document',
                'label_attr' => ['class' => 'control-label'],
            ]);
*/
       /*     $this->addAfter('upload_document', 'extra_image', 'mediaImage', [
                'label'      => 'Extra Image',
                'label_attr' => ['class' => 'control-label'],
            ]);*/
         /*   $this->addAfter('content', 'custom_ipdate', 'date', [
                'label'      => 'Set Custom Publish Date',
                'label_attr' => ['class' => 'control-label'],
                'attr'          => [
                    'class'            => 'form-control datepicker',
                    'data-date-format' => 'yyyy-mm-dd',
                ],
                'default_value' => now(config('app.timezone'))->format('Y-m-d'),

            ]);
            $this->addAfter('custom_ipdate', 'custom_iptime', 'time', [
                'label'      => 'Set Custom Publish Time',
                'label_attr' => ['class' => 'control-label'],

            ]);
            $this->addAfter('custom_ipdate', 'custom_iptime_one', 'time', [
                'label'      => 'Set Custom Publish Time one',
                'label_attr' => ['class' => 'control-label'],

            ]);
          
            $this->addBefore('custom_ipdate', 'extra_new_desc', 'text', [
                'label'      => 'extra content company name/webinar etc',
                'label_attr' => ['class' => 'control-label'],

            ]);

             $this->addAfter('extra_new_desc', 'custom_post_inslider', 'text', [
                        'label'      => 'Enter 1 Show post in Blog slider',
                        'label_attr' => ['class' => 'control-label'],
        
                    ]);
                    
           $this->addBefore('extra_new_desc', 'testimonial_video_url', 'text', [
                        'label'      => 'External Link Video/Pdf',
                        'label_attr' => ['class' => 'control-label'],
        
                    ]);
             $this->addBefore('testimonial_video_url', 'speaker', 'text', [
                        'label'      => 'Speaker',
                        'label_attr' => ['class' => 'control-label'],
        
                    ]);   
            $this->addBefore('testimonial_video_url', 'webinare_evnet', 'text', [
                        'label'      => 'Webinare Event Time',
                        'label_attr' => ['class' => 'control-label'],
        
                    ]);   */          
        }
    }
}
