<?php

namespace Botble\Careers\Models;

use Botble\ACL\Models\User;
use Botble\Base\Traits\EnumCastable;
use Botble\Base\Enums\BaseStatusEnum;
use Botble\Revision\RevisionableTrait;
use Botble\Base\Models\BaseModel;
use Illuminate\Database\Eloquent\Relations\BelongsTo;
use Illuminate\Database\Eloquent\Relations\BelongsToMany;
use Illuminate\Database\Eloquent\Relations\MorphTo;

class Career extends BaseModel
{
    use RevisionableTrait;
    use EnumCastable;

    /**
     * The database table used by the model.
     *
     * @var string
     */
    protected $table = 'careers';

    /**
     * @var mixed
     */
    protected $revisionEnabled = true;

    /**
     * @var mixed
     */
    protected $revisionCleanup = true;

    /**
     * @var int
     */
    protected $historyLimit = 20;

    /**
     * @var array
     */
    protected $dontKeepRevisionOf = [
        'content',
        'views',
       //  'testimonial_video_url',
       // 'extra_new_desc'
        
    ];

    /**
     * The date fields for the model.clear
     *
     * @var array
     */
    protected $dates = [
        'created_at',
        'updated_at',
       // 'extra_new_desc',
        //'custom_ipdate',
        //'custom_iptime'
    ];

    /**
     * The attributes that are mass assignable.
     *
     * @var array
     */
    protected $fillable = [
        'name',
        'department',
        'education',
        'role',
        'experience',
        'description',
        'employmenttype',
        'location',
        'skills',
        'knowledge',
        'requirements',
        'responsibilities',
        'content',
        'image',
        'uploaddocument',
        'extra_image',
        'custom_ipdate',
        'custom_iptime',
        'is_featured',
        'format_type',
        'status',
        'author_id',
        'author_type',
        'post_type',
        'custom_post_inslider',
        'testimonial_video_url',
        'extra_new_desc',
        'speaker',
        'webinare_evnet'
    ];

    /**
     * @var array
     */
    protected $casts = [
        'status' => BaseStatusEnum::class,
    ];

    /**
     * @deprecated
     * @return BelongsTo
     */
    public function user(): BelongsTo
    {
        return $this->belongsTo(User::class)->withDefault();
    }

    /**
     * @return BelongsToMany
     */
    public function tags(): BelongsToMany
    {
        return $this->belongsToMany(Tag::class, 'careers_tags');
    }

    /**
     * @return BelongsToMany
     */
    public function categories(): BelongsToMany
    {
        return $this->belongsToMany(Category::class, 'careers_categories');
    }

    /**
     * @return Category
     */
    public function getFirstCategoryAttribute()
    {
        return $this->categories->first();
    }

    /**
     * @return MorphTo
     */
    public function author(): MorphTo
    {
        return $this->morphTo()->withDefault();
    }

    protected static function boot()
    {
        parent::boot();

        static::deleting(function (Career $career) {
            $career->categories()->detach();
            $career->tags()->detach();
        });
    }
}
