<?php

namespace Botble\Documents\Providers;

use Botble\LanguageAdvanced\Supports\LanguageAdvancedManager;
use Botble\Shortcode\View\View;
use Illuminate\Routing\Events\RouteMatched;
use Botble\Base\Traits\LoadAndPublishDataTrait;
//use Botble\Documents\Models\Post;
use Botble\Documents\Models\Document;
//use Botble\Documents\Repositories\Caches\PostCacheDecorator;
use Botble\Documents\Repositories\Caches\DocumentCacheDecorator;
//use Botble\Documents\Repositories\Eloquent\PostRepository;
use Botble\Documents\Repositories\Eloquent\DocumentRepository;
//use Botble\Documents\Repositories\Interfaces\PostInterface;
use Botble\Documents\Repositories\Interfaces\DocumentInterface;
use Illuminate\Support\Facades\Event;
use Illuminate\Support\ServiceProvider;
use Botble\Documents\Models\Category;
use Botble\Documents\Repositories\Caches\CategoryCacheDecorator;
use Botble\Documents\Repositories\Eloquent\CategoryRepository;
use Botble\Documents\Repositories\Interfaces\CategoryInterface;
use Botble\Documents\Models\Tag;
use Botble\Documents\Repositories\Caches\TagCacheDecorator;
use Botble\Documents\Repositories\Eloquent\TagRepository;
use Botble\Documents\Repositories\Interfaces\TagInterface;
use Language;
use Note;
use SeoHelper;
use SlugHelper;

/**
 * @since 02/07/2016 09:50 AM
 */
class DocumentsServiceProvider extends ServiceProvider
{
    use LoadAndPublishDataTrait;

    public function register()
    {
        $this->app->bind(DocumentInterface::class, function () {
            return new DocumentCacheDecorator(new DocumentRepository(new Document));
        });
         

        $this->app->bind(CategoryInterface::class, function () {
            return new CategoryCacheDecorator(new CategoryRepository(new Category));
        });

        $this->app->bind(TagInterface::class, function () {
            return new TagCacheDecorator(new TagRepository(new Tag));
        });
    }

    public function boot()
    {
        SlugHelper::registerModule(Document::class, 'Documents Posts');
        SlugHelper::registerModule(Category::class, 'Documents Categories');
        SlugHelper::registerModule(Tag::class, 'Documents Tags');

        SlugHelper::setPrefix(Tag::class, 'tag', true);
        SlugHelper::setPrefix(Document::class, null, true);
        SlugHelper::setPrefix(Category::class, null, true);

        $this->setNamespace('plugins/documents')
            ->loadHelpers()
            ->loadAndPublishConfigurations(['permissions', 'general'])
            ->loadAndPublishViews()
            ->loadAndPublishTranslations()
            ->loadRoutes(['web', 'api'])
            ->loadMigrations()
            ->publishAssets();

         $this->app->register(EventServiceProvider::class);

        Event::listen(RouteMatched::class, function () {
            dashboard_menu()
                ->registerItem([
                    'id'          => 'cms-plugins-documents',
                    'priority'    => 3,
                    'parent_id'   => null,
                    'name'        => 'plugins/documents::base.menu_name',
                    'icon'        => 'fa fa-edit',
                    //'url'       => route('posts.index'),
                    'url'         => url("admin/documents/documentsposts"),
                    'permissions' => ['documentsposts.index'],
                ])
                ->registerItem([
                    'id'          => 'cms-plugins-documents-documentsposts',
                    'priority'    => 1,
                    'parent_id'   => 'cms-plugins-documents',
                    'name'        => 'plugins/documents::documentsposts.menu_name',
                    'icon'        => null,
                    //'url'         => route('posts.index'),
                    'url'         => url("admin/documents/documentsposts"),
                    'permissions' => ['documentsposts.index'],
                ])
                ->registerItem([
                    'id'          => 'cms-plugins-documents-categories',
                    'priority'    => 2,
                    'parent_id'   => 'cms-plugins-documents',
                    'name'        => 'plugins/documents::categories.menu_name',
                    'icon'        => null,
                    //'url'         => route('categories.index'),
                    'url'         => url("admin/documents/categories"),
                    'permissions' => ['categories.index'],
                ])
                ->registerItem([
                    'id'          => 'cms-plugins-documents-tags',
                    'priority'    => 3,
                    'parent_id'   => 'cms-plugins-documents',
                    'name'        => 'plugins/documents::tags.menu_name',
                    'icon'        => null,
                    'url'         => url("admin/documents/tags"),
                    //'url'         => route('tags.index'),
                    'permissions' => ['tags.index'],
                ]);
        });

        $useLanguageV2 = $this->app['config']->get('plugins.documents.general.use_language_v2', false) &&
            defined('LANGUAGE_ADVANCED_MODULE_SCREEN_NAME');

        if (defined('LANGUAGE_MODULE_SCREEN_NAME') && $useLanguageV2) {
            LanguageAdvancedManager::registerModule(Document::class, [
                'name',
                'description',
                'content',
            ]);

            LanguageAdvancedManager::registerModule(Category::class, [
                'name',
                'description',
            ]);

            LanguageAdvancedManager::registerModule(Tag::class, [
                'name',
                'description',
            ]);
        }

        $this->app->booted(function () use ($useLanguageV2) {
            $models = [Document::class, Category::class, Tag::class];

            if (defined('LANGUAGE_MODULE_SCREEN_NAME') && !$useLanguageV2) {
                Language::registerModule($models);
            }

            SeoHelper::registerModule($models);

            $configKey = 'packages.revision.general.supported';
            config()->set($configKey, array_merge(config($configKey, []), [Document::class]));

            if (defined('NOTE_FILTER_MODEL_USING_NOTE')) {
                Note::registerModule(Document::class);
            }

            $this->app->register(HookServiceProvider::class);
        });

        if (function_exists('shortcode')) {
            view()->composer([
                'plugins/documents::themes.documents',
                'plugins/documents::themes.category',
                'plugins/documents::themes.tag',
            ], function (View $view) {
                $view->withShortcodes();
            });
        }
    }
}
