<?php

namespace Botble\Careers\Providers;

use Assets;
use BaseHelper;
use Botble\Base\Enums\BaseStatusEnum;
use Botble\Base\Models\BaseModel;
use Botble\Careers\Models\Category;
use Botble\Careers\Models\Career;
use Botble\Careers\Models\Tag;
use Botble\Careers\Services\CareersService;
use Botble\Careers\Repositories\Interfaces\CareerInterface;
use Botble\Dashboard\Supports\DashboardWidgetInstance;
use Botble\Page\Models\Page;
use Botble\Page\Repositories\Interfaces\PageInterface;
use Eloquent;
use Html;
use Illuminate\Routing\Events\RouteMatched;
use Illuminate\Support\Collection;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Event;
use Illuminate\Support\ServiceProvider;
use Illuminate\Support\Str;
use Menu;
use RvMedia;
use stdClass;
use Theme;

class HookServiceProvider extends ServiceProvider
{
    public function boot()
    {
        add_filter(DASHBOARD_FILTER_ADMIN_LIST, [$this, 'addCareerStatsWidget'], 12, 2);
        add_filter(DASHBOARD_FILTER_ADMIN_LIST, [$this, 'registerDashboardWidgets'], 21, 2);
        if (defined('MENU_ACTION_SIDEBAR_OPTIONS')) {
            Menu::addMenuOptionModel(Category::class);
            Menu::addMenuOptionModel(Tag::class);
            add_action(MENU_ACTION_SIDEBAR_OPTIONS, [$this, 'registerMenuOptions'], 2);
        }
       
        add_filter(BASE_FILTER_PUBLIC_SINGLE_DATA, [$this, 'handleSingleView'], 2);
        if (defined('PAGE_MODULE_SCREEN_NAME')) {
            add_filter(PAGE_FILTER_FRONT_PAGE_CONTENT, [$this, 'renderCareersPage'], 2, 2);
            add_filter(PAGE_FILTER_PAGE_NAME_IN_ADMIN_LIST, [$this, 'addAdditionNameToPageName'], 147, 2);
        }

        Event::listen(RouteMatched::class, function () {
            if (function_exists('admin_bar')) {
                Theme::composer('*', function () {
                    if (Auth::check() && Auth::user()->hasPermission('careersposts.create')) {
                       
                       admin_bar()->registerLink(trans('plugins/careers::careersposts.career'), route('careersposts.create'), 'add-new');

                    }
                });
            }
        });

        if (function_exists('add_shortcode')) {
            add_shortcode('careers-careersposts', trans('plugins/careers::base.short_code_name'),
                trans('plugins/careers::base.short_code_description'), [$this, 'renderCareersposts']);
            shortcode()->setAdminConfig('careers-careersposts', function ($attributes, $content) {
                return view('plugins/careers::partials.careerspostss-short-code-admin-config', compact('attributes', 'content'))
                    ->render();
            });
        }

        if (function_exists('theme_option')) {
            add_action(RENDERING_THEME_OPTIONS_PAGE, [$this, 'addThemeOptions'], 35);
        }

        if (defined('LANGUAGE_MODULE_SCREEN_NAME')) {
            add_action(BASE_ACTION_META_BOXES, [$this, 'addLanguageChooser'], 55, 2);
        }

        if (defined('THEME_FRONT_HEADER')) {
            add_action(BASE_ACTION_PUBLIC_RENDER_SINGLE, function ($screen, $career) {
                add_filter(THEME_FRONT_HEADER, function ($html) use ($career) {
                    if (get_class($career) != Careers::class) {
                        return $html;
                    }

                    $schema = [
                        '@context'         => 'https://schema.org',
                        '@type'            => 'NewsArticle',
                        'mainEntityOfPage' => [
                            '@type' => 'WebPage',
                            '@id'   => $career->url,
                        ],
                        'headline'         => BaseHelper::clean($career->name),
                        'description'      => BaseHelper::clean($career->description),
                        'image'            => [
                            '@type' => 'ImageObject',
                            'url'   => RvMedia::getImageUrl($career->image, null, false, RvMedia::getDefaultImage()),
                        ],
                         'uploadcareer'            => [
                            '@type' => 'uploadcareerObject',
                            'url'   => RvMedia::getuploadcareerUrl($career->uploadcareer, null, false, RvMedia::getDefaultuploadcareer()),
                        ],
                        'author'           => [
                            '@type' => 'Person',
                            'url'   => route('public.index'),
                            'name'  => $career->author->name,
                        ],
                        'publisher'        => [
                            '@type' => 'Organization',
                            'name'  => theme_option('site_title'),
                            'logo'  => [
                                '@type' => 'ImageObject',
                                'url'   => RvMedia::getImageUrl(theme_option('logo')),
                            ],
                        ],
                        'datePublished'    => $career->created_at->toDateString(),
                        'dateModified'     => $career->updated_at->toDateString(),
                    ];

                    return $html . Html::tag('script', json_encode($schema), ['type' => 'application/ld+json'])
                            ->toHtml();
                }, 35);
            }, 35, 2);
        }
    }

    public function addThemeOptions()
    {
        $pages = $this->app->make(PageInterface::class)->pluck('name', 'id', ['status' => BaseStatusEnum::PUBLISHED]);

        theme_option()
            ->setSection([
                'title'      => 'Careers',
                'desc'       => 'Theme options for careers',
                'id'         => 'opt-text-subsection-careers',
                'subsection' => true,
                'icon'       => 'fa fa-edit',
                'fields'     => [
                    [
                        'id'         => 'careersposts_page_id',
                        'type'       => 'customSelect',
                        'label'      => trans('plugins/careers::base.careersposts_page_id'),
                        'attributes' => [
                            'name'    => 'careersposts_page_id',
                            'list'    => ['' => trans('plugins/careers::base.select')] + $pages,
                            'value'   => '',
                            'options' => [
                                'class' => 'form-control',
                            ],
                        ],
                    ],
                    [
                        'id'         => 'number_of_careersposts_in_a_category',
                        'type'       => 'number',
                        'label'      => trans('plugins/careers::base.number_careersposts_per_page_in_category'),
                        'attributes' => [
                            'name'    => 'number_of_careersposts_in_a_category',
                            'value'   => 12,
                            'options' => [
                                'class' => 'form-control',
                            ],
                        ],
                    ],
                    [
                        'id'         => 'number_of_careersposts_in_a_tag',
                        'type'       => 'number',
                        'label'      => trans('plugins/careers::base.number_careersposts_per_page_in_tag'),
                        'attributes' => [
                            'name'    => 'number_of_careersposts_in_a_tag',
                            'value'   => 12,
                            'options' => [
                                'class' => 'form-control',
                            ],
                        ],
                    ],
                ],
            ]);
    }

    /**
     * Register sidebar options in menu
     */
    public function registerMenuOptions()
    {
        if (Auth::user()->hasPermission('categories.index')) {
            Menu::registerMenuOptions(Category::class, trans('plugins/careers::categories.menu'));
        }

        if (Auth::user()->hasPermission('tags.index')) {
            Menu::registerMenuOptions(Tag::class, trans('plugins/careers::tags.menu'));
        }
    }

    /**
     * @param array $widgets
     * @param Collection $widgetSettings
     * @return array
     */
    public function registerDashboardWidgets($widgets, $widgetSettings)
    {
        // if (!Auth::user()->hasPermission('careersposts.index')) {
        //     return $widgets;
        // }

        Assets::addScriptsDirectly(['/vendor/core/plugins/careers/js/blog.js']);

        return (new DashboardWidgetInstance)
            ->setPermission('careersposts.index')
            ->setKey('widget_careersposts_recent')
            ->setTitle(trans('Recent Careers Posts'))
            ->setIcon('fas fa-edit')
            ->setColor('#f3c200')
            ->setRoute(route('careersposts.widget.recent-careersposts'))
            ->setBodyClass('scroll-table')
            ->setColumn('col-md-6 col-sm-6')
            ->init($widgets, $widgetSettings);
    }

    public function addCareerStatsWidget($widgets, $widgetSettings)
    {
        $careers = $this->app->make(CareerInterface::class)->count();

        return (new DashboardWidgetInstance)
            ->setType('stats')
            ->setPermission('careers.index')
            ->setTitle(trans('Jobs'))
            ->setKey('widget_total_careers')
            ->setIcon('fas fa-book')
            ->setColor('#3598dc')
            ->setStatsTotal($careers)
            ->setRoute(route('careersposts.widget.recent-careersposts'))
            ->init($widgets, $widgetSettings);
    }

    /**
     * @param Eloquent $slug
     * @return array|Eloquent
     */
    public function handleSingleView($slug)
    {
        return (new CareersService)->handleFrontRoutes($slug);
    }

    /**
     * @param stdClass $shortcode
     * @return array|string
     */
    public function renderCareersPosts($shortcode)
    {
        $career = get_all_careersposts(true, $shortcode->paginate, ['slugable', 'categories', 'categories.slugable', 'author']);

        $view = 'plugins/careers::themes.templates.careersposts';
        $themeView = Theme::getThemeNamespace() . '::views.templates.careersposts';
        if (view()->exists($themeView)) {
            $view = $themeView;
        }

        return view($view, compact('careersposts'))->render();
    }

    /**
     * @param string|null $content
     * @param Page $page
     * @return array|string|null
     */
    public function renderCareersPage(?string $content, Page $page)
    {
        if ($page->id == theme_option('careersposts_page_id', setting('careersposts_page_id'))) {
            $view = 'plugins/careers::themes.loop';

            if (view()->exists(Theme::getThemeNamespace() . '::views.loop')) {
                $view = Theme::getThemeNamespace() . '::views.loop';
            }

            return view($view, [
                'careersposts' => get_all_careersposts(
                    true,
                    theme_option('number_of_careersposts_in_a_category', 12),
                    ['slugable', 'categories', 'categories.slugable', 'author']
                ),
            ])
                ->render();
        }

        return $content;
    }

    /**
     * @param string|null $name
     * @param Page $page
     * @return string|null
     */
    public function addAdditionNameToPageName(?string $name, Page $page)
    {
        if ($page->id == theme_option('careersposts_page_id', setting('careers_page_id'))) {
            $subTitle = Html::tag('span', trans('plugins/careers::base.career_page'), ['class' => 'additional-page-name'])
                ->toHtml();

            if (Str::contains($name, ' —')) {
                return $name . ', ' . $subTitle;
            }

            return $name . ' —' . $subTitle;
        }

        return $name;
    }

    /**
     * @param BaseModel $model
     * @param string $priority
     * @return string
     */
    public function addLanguageChooser($priority, $model)
    {
        if ($priority == 'head' && $model instanceof Category) {

            $route = 'categories.index';

            if ($route) {
                echo view('plugins/language::partials.admin-list-language-chooser', compact('route'))->render();
            }
        }
    }
}
